
--packages
PlayerQuest.events = {};


PlayerQuest.events.UpdateData = {}
PlayerQuest.events.nUpdates = 0;


function PlayerQuest.events.RegisterEvents()
	this:RegisterEvent("MINIMAP_ZONE_CHANGED");
	this:RegisterEvent("ZONE_CHANGED_NEW_AREA");
	this:RegisterEvent("CHAT_MSG_COMBAT_HOSTILE_DEATH");
	this:RegisterEvent("CHAT_MSG_COMBAT_PARTY_HITS");
	this:RegisterEvent("CHAT_MSG_COMBAT_PET_HITS");
	this:RegisterEvent("CHAT_MSG_COMBAT_SELF_HITS"); 
	this:RegisterEvent("CHAT_MSG_COMBAT_XP_GAIN");

	this:RegisterEvent("PLAYER_DEAD");
end

function PlayerQuest.events.CheckCompletedQuest(iii)
	-- Check to see if this quest is finished...
	local pdata = PlayerQuest.PlayerData();
	for jjj = 1, pdata.quests[iii].nLeaderBoards, 1 do
		if((not pdata.quests[iii].leaderBoards[jjj].isComplete) or (pdata.quests[iii].isFailed)) then
			return;
		end
	end
	
	pdata.quests[iii].isComplete = 1;
end

function PlayerQuest.events.FailQuest(iii)
	local pdata = PlayerQuest.PlayerData();
	if(not pdata.quests[iii].isFailed) then
		PlayerQuest.OutputMessage("Quest ("..pdata.quests[iii].titleText..") failed.");
		PlayerQuest.CompMessage("Quest ("..pdata.quests[iii].titleText..") failed.");
	end
	pdata.quests[iii].isFailed = 1;
	pdata.quests[iii].isComplete = nil;
end


function PlayerQuest.events.CheckLev(req, c_data)
	if((req.levelRangeLow ~= 0) and (req.levelRangeLow > c_data.level)) then
		return nil;
	end
	
	if((req.levelRangeHigh ~= 0) and (req.levelRangeHigh < c_data.level)) then
		return nil;
	end
	
	return 1;
end

function PlayerQuest.events.AddKillCheck(c_data)
	local pdata = PlayerQuest.PlayerData();
	
	for i = 1, pdata.nQuests, 1 do
		for j = 1, pdata.quests[i].nRequired, 1 do
			if( pdata.quests[i].required[j].type == "monster") then
				if( pdata.quests[i].required[j].name == c_data.name) then
					PlayerQuest.events.AddKill(i, j);
				end
			end
			if( pdata.quests[i].required[j].type == "pvp_any") then
				if(c_data.pvp and PlayerQuest.events.CheckLev(pdata.quests[i].required[j], c_data)) then
					PlayerQuest.events.AddKill(i, j);
				end
			end
			if( pdata.quests[i].required[j].type == "pvp_guild") then
				if(c_data.guild and PlayerQuest.events.CheckLev(pdata.quests[i].required[j], c_data)) then
					if(c_data.guild == pdata.quests[i].required[j].name) then
						PlayerQuest.events.AddKill(i, j);
					end
				end
			end
			if( pdata.quests[i].required[j].type == "pvp_specific") then
				if(c_data.name and PlayerQuest.events.CheckLev(pdata.quests[i].required[j], c_data)) then
					if(c_data.name == pdata.quests[i].required[j].name) then
						PlayerQuest.events.AddKill(i, j);
					end
				end
			end
			if( pdata.quests[i].required[j].type == "pvp_race") then
				if(c_data.race and PlayerQuest.events.CheckLev(pdata.quests[i].required[j], c_data)) then
					if(c_data.race == pdata.quests[i].required[j].name) then
						PlayerQuest.events.AddKill(i, j);
					end
				end
			end
			if( pdata.quests[i].required[j].type == "pvp_class") then
				if(c_data.class and PlayerQuest.events.CheckLev(pdata.quests[i].required[j], c_data)) then
					if(c_data.class == pdata.quests[i].required[j].name) then
						PlayerQuest.events.AddKill(i, j);
					end
				end
			end
		end
	end
end

function PlayerQuest.events.AddKill(ii, jj)
	local pdata = PlayerQuest.PlayerData();
	
	local lb = pdata.quests[ii].required[jj].leaderBoard
	if(pdata.quests[ii].leaderBoards[lb].isComplete) then
		return;
	end;
	pdata.quests[ii].leaderBoards[lb].countNumber  = pdata.quests[ii].leaderBoards[lb].countNumber + 1;
	if(pdata.quests[ii].leaderBoards[lb].countNumber == pdata.quests[ii].leaderBoards[lb].countMax) then
		pdata.quests[ii].leaderBoards[lb].isComplete = 1;
	end;
	
	PlayerQuest.CompMessage(PlayerQuest.questlog.PrintLeaderBoard(lb, ii));
	PlayerQuest.events.CheckCompletedQuest(ii);
	PlayerQuest.events.UpdateOnEvents("QUEST_LOG_UPDATE");
end;

function PlayerQuest.events.AddDuelWin(ii, jj)
	local pdata = PlayerQuest.PlayerData();
	
	local lb = pdata.quests[ii].required[jj].leaderBoard
	if(pdata.quests[ii].leaderBoards[lb].isComplete) then
		return;
	end;
	pdata.quests[ii].leaderBoards[lb].countNumber  = pdata.quests[ii].leaderBoards[lb].countNumber + 1;
	if(pdata.quests[ii].leaderBoards[lb].countNumber == pdata.quests[ii].leaderBoards[lb].countMax) then
		pdata.quests[ii].leaderBoards[lb].isComplete = 1;
	end;
	
	PlayerQuest.CompMessage(PlayerQuest.questlog.PrintLeaderBoard(lb, ii));
	PlayerQuest.events.CheckCompletedQuest(ii);
	PlayerQuest.events.UpdateOnEvents("QUEST_LOG_UPDATE");
end;

function PlayerQuest.events.CompleteObjective(ii, jj)
	local pdata = PlayerQuest.PlayerData();
	
	local lb = pdata.quests[ii].required[jj].leaderBoard
	if(pdata.quests[ii].leaderBoards[lb].isComplete) then
		return;
	end;
	pdata.quests[ii].leaderBoards[lb].isComplete = 1;
	PlayerQuest.CompMessage(pdata.quests[ii].leaderBoards[lb].text.." Complete");
	PlayerQuest.events.CheckCompletedQuest(ii);
	PlayerQuest.events.UpdateOnEvents("QUEST_LOG_UPDATE");
end;

function PlayerQuest.events.ProcessEvents(event)
	if(PlayerQuest.Ready() == nil) then
		return;
	end;
	
	local pdata = PlayerQuest.PlayerData();

	if(event == "PLAYER_TARGET_CHANGED") then
		PQTBadge1:Hide();
		PQTBadge2:Hide();
		local uname = UnitName("target");
		if(uname == nil) then
			return;
		end

		local PQ_PlayerClass, PQ_PlayerRace, PQ_PlayerLevel, PQ_PlayerGuild = UnitClass("player"), UnitRace("player"), UnitLevel("player"), GetGuildInfo("player");

		if(PQ_PlayerGuild == nil) then
			PQ_PlayerGuild = ""
		end;
		-- PlayerQuest.reputation.UpdateRep(UnitName("target"));
		PlayerQuest.comms.Send(UnitName("target"), "MSG_CHECK_QUESTS", PQ_PlayerClass.."@"..PQ_PlayerRace.."@"..PQ_PlayerLevel.."@"..PQ_PlayerGuild);

		return;
	end
	
	if(event == "PLAYER_DEAD") then
		for i = 1, pdata.nQuests, 1 do
			for j = 1, pdata.quests[i].nFailureConditions, 1 do
				if( pdata.quests[i].failureConditions[j].type == "death") then
					PlayerQuest.events.FailQuest(i);
				end
			end
		end
		return;
	end;
	
	if((event == "MINIMAP_ZONE_CHANGED") or (event == "ZONE_CHANGED_NEW_AREA")) then
	    local szone=GetSubZoneText();
	    local zone=GetZoneText();
		
		-- check through all reqs to see if you can find any needed updates...
		for i = 1, pdata.nQuests, 1 do
			for j = 1, pdata.quests[i].nRequired, 1 do
				if( pdata.quests[i].required[j].type == "area") then
					if( pdata.quests[i].required[j].name == zone) then
						PlayerQuest.events.CompleteObjective(i, j);
					end
					if( szone ) then
						if( pdata.quests[i].required[j].name == szone) then
							PlayerQuest.events.CompleteObjective(i, j);
						end
					end;
				end
			end
		end
	else
		local s, e, creep = string.find(arg1, "(.+) dies");
		if(not creep) then
			prevCreepName = nil;
			return;
		end;
		
		local c_data = {};
		
		
		local s2, e2, xp = string.find(arg1, "gain (.+) experience");
		if(xp and (not prevCreepName)) then
--			PlayerQuest.OutputMessage("You gained "..xp.." experience for killing "..creep..".");
			if(UnitName("target") ~= creep) then
--				PlayerQuest.OutputMessage(creep.." was killed by you.");
				-- Add a kill
				-- This one only happens for non-targetted NPC's
				c_data.name = creep;
				c_data.pvp = nil;
				c_data.guild = nil;
				c_data.race = nil;
				c_data.class = nil;
				c_data.faction = nil;
				c_data.level = 0;
				PlayerQuest.events.AddKillCheck(c_data);
			end
		end
		
--		PlayerQuest.OutputMessage(creep.." found dying...");
		
		if(UnitIsTapped("target")) then
			if(UnitIsTappedByPlayer("target")) then
				if((UnitHealth("target") <= 0) and (UnitName("target") == creep)) then
--					PlayerQuest.OutputMessage(UnitName("target").." was killed by you.");
					-- Add a kill
					c_data.name = creep;
					c_data.pvp = UnitIsPVP("target");
					c_data.guild = GetGuildInfo("target");
					c_data.race = UnitRace("target");
					c_data.class = UnitClass("target");
					c_data.faction = UnitFactionGroup("target");
					c_data.level = UnitLevel("target");
					PlayerQuest.events.AddKillCheck(c_data);
					prevCreepName = creep;
				end
--			else
--				PlayerQuest.OutputMessage("Target is tapped.");
			end;
		else
			if((UnitHealth("target") <= 0) and (UnitName("target") == creep)) then
--				PlayerQuest.OutputMessage(UnitName("target").." was killed by you.");
				--Detect kill!
				c_data.name = creep;
				c_data.pvp = UnitIsPlayer("target");
				c_data.guild = GetGuildInfo("target");
				c_data.race = UnitRace("target");
				c_data.class = UnitClass("target");
				c_data.faction = UnitFactionGroup("target");
				c_data.level = UnitLevel("target");
				PlayerQuest.events.AddKillCheck(c_data);
				prevCreepName = creep;
			end
		end
		
	end
end


PlayerQuest.events.LastMsg = nil;

function ChatFrame_OnEvent()
    -- exit if not inited
    if( not PlayerQuest.Ready() ) then
		PlayerQuest.overrides.ChatFrame_OnEvent(event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9);
        return;
    end
    --PlayerQuest.CompMessage(event);
    --error spam...lol
   
    if(((event=="CHAT_MSG_CHANNEL_JOIN") or (event=="CHAT_MSG_CHANNEL_LEAVE")) and PlayerQuest.comms.GetChannelNum()) then
    	--PlayerQuest.CompMessage("1:"..arg1.."2:"..arg2);
    	if(arg8==PlayerQuest.comms.GetChannelNum()) then
    		-- ignore join/leave spam...
    		return;
    	end
    end

   	if(PlayerQuest.events.LastMsg ~= arg1) then
    	if((event=="CHAT_MSG_CHANNEL") and (arg8==PlayerQuest.comms.GetChannelNum())) then
    		--PlayerQuest.OutputMessage("test: "..arg1.."~="..PlayerQuest.events.LastMsg);
    		if(PlayerQuest.comms.CheckMsg(arg1, arg2)) then
    			if(arg1 ~= nil) then
    				PlayerQuest.events.LastMsg = arg1;
    			end
    			return;
    		end;
    	end;
    else
    	if(arg8==PlayerQuest.comms.GetChannelNum()) then
   			return;
    	end;
    end;
	PlayerQuest.overrides.ChatFrame_OnEvent(event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9);

    
    --PlayerQuest.OutputMessage(arg8);
    local pdata = PlayerQuest.PlayerData();

    if( event and arg1 ) then
        local s, e, winner, loser;
        local player = PlayerQuest.ShortPlayerName;
        s, e, winner, loser = string.find(arg1, "(.+) has defeated (.+) in a duel");
        if( winner ) then
            --- CHAT_MSG_SYSTEM
            if( player == winner ) then
                local v = { };
                v.name = loser;
                TargetByName(loser);
                if( UnitName("target") == loser ) then
                    v.level = UnitLevel("target");
                    v.class = UnitClass("target");
                    v.guild = GetGuildInfo("target");
                    v.race = UnitRace("target");
                else
                    v.level = -1;
                end

				-- check through for kill events...
				for i = 1, pdata.nQuests, 1 do
					for j = 1, pdata.quests[i].nRequired, 1 do
						if( pdata.quests[i].required[j].type == "duel_specific") then
							if( pdata.quests[i].required[j].name == v.name) then
								if( PlayerQuest.events.CheckLev(pdata.quests[i].required[j], v)) then
									PlayerQuest.events.AddDuelWin(i, j);
								end
							end
						end
						if( pdata.quests[i].required[j].type == "duel_any") then
							if( PlayerQuest.events.CheckLev(pdata.quests[i].required[j], v)) then
								PlayerQuest.events.AddDuelWin(i, j);
							end
						end
						if( pdata.quests[i].required[j].type == "duel_guild") then
							if( pdata.quests[i].required[j].name == v.guild) then
								if( PlayerQuest.events.CheckLev(pdata.quests[i].required[j], v)) then
									PlayerQuest.events.AddDuelWin(i, j);
								end
							end
						end
						if( pdata.quests[i].required[j].type == "duel_class") then
							if( pdata.quests[i].required[j].name == v.class) then
								if( PlayerQuest.events.CheckLev(pdata.quests[i].required[j], v)) then
									PlayerQuest.events.AddDuelWin(i, j);
								end
							end
						end
					end
				end
            end
        end
    end

	PlayerQuest.events.LastMsg = arg1;
end

PlayerQuest.events.PrevTime = -1.0;

function PlayerQuest.OnUpdate()
	--check if we're loaded
	if(not PlayerQuest.Ready()) then
		return;
	end
	
	local pdata = PlayerQuest.PlayerData();
	
	--return if there is no buff up
	local ttable = pdata.quests;
	local elapsed = 1.0;
	if(PlayerQuest.events.PrevTime == -1.0) then
		PlayerQuest.events.PrevTime = (time());
		elapsed = 0.0;
	else
		elapsed = (time()) - PlayerQuest.events.PrevTime;
		--PlayerQuest.OutputMessage("PrevTime : "..PlayerQuest.events.PrevTime.." New Time : "..(time()));
		PlayerQuest.events.PrevTime = (time());
	end
	for key, value in ttable do
		if((value.timeLeft ~= 0) and (value.timeLeft ~= nil)) then
			value.timeLeft = value.timeLeft - elapsed;
			if(value.timeLeft <= 0) then
				PlayerQuest.events.FailQuest(key);
				value.timeLeft = 0;
			end
			--PlayerQuest.events.UpdateOnEvents("QUEST_LOG_UPDATE");
		end
		
		--check items...
		for keyr, valuer in value.required do
			if(valuer.type == "item") then
				--check inventory
				local count = PlayerQuest.events.InventoryCheck(valuer.link);
				
				--PlayerQuest.OutputMessage("Number found: "..count);
				--print a notice if its different.
				if(count > value.leaderBoards[valuer.leaderBoard].countMax) then
					count = value.leaderBoards[valuer.leaderBoard].countMax;
				end
				if(value.leaderBoards[valuer.leaderBoard].countNumber ~= count) then
					PlayerQuest.CompMessage(value.leaderBoards[valuer.leaderBoard].text..": "..count.."/"..value.leaderBoards[valuer.leaderBoard].countMax);
					PlayerQuest.events.UpdateOnEvents("QUEST_LOG_UPDATE");
					value.isComplete = nil;
					PlayerQuest.events.CheckCompletedQuest(key);
				end
				
				--set the leaderboard
				value.leaderBoards[valuer.leaderBoard].countNumber = count;
				if(count == value.leaderBoards[valuer.leaderBoard].countMax) then
					PlayerQuest.events.UpdateOnEvents("QUEST_LOG_UPDATE");
					value.leaderBoards[valuer.leaderBoard].isComplete = 1;
					PlayerQuest.events.CheckCompletedQuest(key);
				end
				
			end
		end
	end
end

function PlayerQuest.events.InventoryCheck(link)
	if(link == nil) then
		return 0;
	end;
	local count = 0;
	for i=0, 4, 1 do
		for j=0, 16, 1 do
			if(link == GetContainerItemLink(i, j)) then
				local texture, itemCount, locked, quality, readable = GetContainerItemInfo(i, j);
				count = count + itemCount;
			end
		end
	end
	
	return count;
end

function PlayerQuest.events.RegisterUpdates(func)
	for i = 1, PlayerQuest.events.nUpdates, 1 do
		if(PlayerQuest.events.UpdateData[i].OnEvent == func) then
			return;
		end;
	end;
	
	PlayerQuest.events.nUpdates = PlayerQuest.events.nUpdates + 1;
	
	PlayerQuest.events.UpdateData[PlayerQuest.events.nUpdates] = {};
	PlayerQuest.events.UpdateData[PlayerQuest.events.nUpdates].OnEvent = func;
end

function PlayerQuest.events.UpdateOnEvents(event)
	-- do the updates
	for i = 1, PlayerQuest.events.nUpdates, 1 do
		PlayerQuest.events.UpdateData[i].OnEvent(event);
	end
end