--
-- Player Quest
--
--
-- Player Created Quests!!! 
-- 
--  Slash Command:
--    /pcq
--

------------------------------------------
-- Packages								--
--  - questlog							--
--		All questlog functions/variables--
------------------------------------------
PlayerQuest.questlog = {}

------------------------------------------
-- Global Variables						--
--  - AbandonQuest						--
--		The quest that is set to be		--
--		Abandoned						--
------------------------------------------
PlayerQuest.questlog.AbandonQuest = nil;
PlayerQuest.questlog.Selection = 0;


function PlayerQuest.questlog.GetSelectedQuest()
	local pdata = PlayerQuest.PlayerData();
	return pdata.quests[PlayerQuest.questlog.Selection];
end

------------------------------------------------------
-- PrintLeaderBoard(leaderBoard, selection)			--
--  Returns: None									--
------------------------------------------------------
function PlayerQuest.questlog.PrintLeaderBoard(i, q)
	local pqs = q;
	if(not pqs) then
		pqs = PlayerQuest.questlog.Selection;
	end
	local ps = "";
	local pdata = PlayerQuest.PlayerData();
	if(pdata.quests[pqs].leaderBoards[i].ptype == "PTYPE_COUNT") then
		 ps = pdata.quests[pqs].leaderBoards[i].text..": "..pdata.quests[pqs].leaderBoards[i].countNumber.."/"..pdata.quests[pqs].leaderBoards[i].countMax;
	end
	if(pdata.quests[pqs].leaderBoards[i].ptype == "PTYPE_TEXT") then
		 ps = pdata.quests[pqs].leaderBoards[i].text;
	end

	return ps;
end

------------------------------------------******Overrides******---------------------------------------

------------------------------------------
--	function AbandonQuest				--
--		Deletes a quest from the log	--
------------------------------------------
function AbandonQuest()
	if(PlayerQuest.questlog.AbandonQuest) then
		local l;
		local pdata = PlayerQuest.PlayerData();
		for l = PlayerQuest.questlog.AbandonQuest, pdata.nQuests, 1 do
			pdata.quests[l] = pdata.quests[l+1];
		end
		pdata.nQuests = pdata.nQuests - 1;

		PlayerQuest.questlog.AbandonQuest = nil;
		PlayerQuest.events.UpdateOnEvents("QUEST_LOG_UPDATE");
		return;
	end

	return PlayerQuest.overrides.AbandonQuest();
end

------------------------------------------
--	function SetAbandonQuest			--
--		prepares to remove a quest		--
------------------------------------------
function SetAbandonQuest()
	PlayerQuest.questlog.AbandonQuest = nil;
	if(PlayerQuest.questlog.Selection > 0) then
		PlayerQuest.questlog.AbandonQuest = PlayerQuest.questlog.Selection;
		return;
	end;
	
	PlayerQuest.overrides.SetAbandonQuest();
end

------------------------------------------
--	function GetAbandonQuestName		--
------------------------------------------
function GetAbandonQuestName()
	local pdata = PlayerQuest.PlayerData();
	
	if(PlayerQuest.questlog.AbandonQuest ~= nil) then
		return pdata.quests[PlayerQuest.questlog.AbandonQuest].titleText;
	end
	
	return PlayerQuest.overrides.GetAbandonQuestName();
end


------------------------------------------------------
-- GetQuestLogPushable()							--
--  returns whether the quest can be pushed			--
------------------------------------------------------
function GetQuestLogPushable()
	if(PlayerQuest.Ready()) then
		local pdata = PlayerQuest.PlayerData();
		if(pdata.nQuests > 0) then
			if(PlayerQuest.questlog.Selection > 0) then
			if(PlayerQuest.questlog.Selection <= pdata.nQuests) then 
				return pdata.quests[PlayerQuest.questlog.Selection].isPushable;
			end
			end
		end
	end

	return PlayerQuest.overrides.GetQuestLogPushable();
end

--------------------------------------------------
-- GetQuestLogQuestText()						--
--  returns description and objectives			--
--------------------------------------------------
function GetQuestLogQuestText()
	if(PlayerQuest.Ready()) then
		local pdata = PlayerQuest.PlayerData();
		if(pdata.nQuests > 0) then
			if(PlayerQuest.questlog.Selection > 0) then
				local sq = PlayerQuest.questlog.GetSelectedQuest();
				return sq.Description, sq.Objectives;
			end
		end
	end

	return PlayerQuest.overrides.GetQuestLogQuestText();
end

--------------------------------------------------
-- GetQuestLogSelection()						--
--  returns the selected quest					--
--------------------------------------------------
function GetQuestLogSelection()
	if(PlayerQuest.questlog.Selection > 0) then
		return PlayerQuest.questlog.Selection + 1;
	end

	if(PlayerQuest.Ready()) then
		local pdata = PlayerQuest.PlayerData();

		if(pdata.nQuests > 0) then
			return PlayerQuest.overrides.GetQuestLogSelection() + pdata.nQuests + 1;
		end
	end

	return PlayerQuest.overrides.GetQuestLogSelection()
end

----------------------------------------------
-- ExpandQuestHeader(questID) 				--
--  Expands a header						--
-- **MOSTLY FINISHED**						--
----------------------------------------------
function ExpandQuestHeader(questID)
	if(((questID == 0) or (questID == 1)) and (PlayerQuest.Ready())) then
		local pdata = PlayerQuest.PlayerData();

		if(questID==0) then
			return PlayerQuest.overrides.ExpandQuestHeader(0);
		else
			if(pdata.nQuests > 0) then
				return;
			end
		end
	end
	if(PlayerQuest.Ready()) then
		local pdata = PlayerQuest.PlayerData();
		if(pdata.nQuests>0) then
			return PlayerQuest.overrides.ExpandQuestHeader(questID - (1 + pdata.nQuests));
		else
			return PlayerQuest.overrides.ExpandQuestHeader(questID);
		end;
	end
end

----------------------------------------------
-- CollapseQuestHeader(questID) 			--
--  Collapses a header						--
-- **MOSTLY FINISHED**						--
----------------------------------------------
function CollapseQuestHeader(questID)
	if(PlayerQuest.Ready()) then
		local pdata = PlayerQuest.PlayerData();
		if((questID == 0) or (questID == 1)) then
			if(questID==0) then
				return PlayerQuest.overrides.CollapseQuestHeader(0);
			else
				if(pdata.nQuests > 0) then
					return;
				end
			end
		end
		if(pdata.nQuests>0) then
			return PlayerQuest.overrides.CollapseQuestHeader(questID - (1 + pdata.nQuests));
		else
			return PlayerQuest.overrides.CollapseQuestHeader(questID);
		end
	end
end


----------------------------------------------
-- GetNumQuestLogEntries()	 				--
--  Gets the number of Entries, Quests		--
----------------------------------------------
function GetNumQuestLogEntries()
	if(PlayerQuest.Ready()) then
		local pdata = PlayerQuest.PlayerData();
		if(pdata.nQuests > 0) then

			local ent, ques = PlayerQuest.overrides.GetNumQuestLogEntries();
			return  ent + 1 + pdata.nQuests, ques + pdata.nQuests;
		end
	end

	return PlayerQuest.overrides.GetNumQuestLogEntries();
end

------------------------------------------------------
-- GetQuestLogTitle(questIndex)						--
--  Returns: questLogTitleText, level, questTag, 	--
--  isHeader, isCollapsed, isComplete				--
------------------------------------------------------
function GetQuestLogTitle(questIndex)
	local questLogTitleText, level, questTag, isHeader, isCollapsed, isComplete;
	
	if(PlayerQuest.Ready()) then
		local pdata = PlayerQuest.PlayerData();
		if(pdata.nQuests > 0) then
			if(questIndex > 1 + pdata.nQuests) then
				return PlayerQuest.overrides.GetQuestLogTitle(questIndex - (1 + pdata.nQuests));
			else
				if(questIndex == 1) then
					return "Player Created", 0, nil, 1, nil, nil;
				end
				questLogTitleText = pdata.quests[questIndex-1].titleText;
				level = pdata.quests[questIndex-1].level;
				questTag = pdata.quests[questIndex-1].questTag;
				isHeader = nil;
				isCollapsed = nil; 
				isComplete = pdata.quests[questIndex-1].isComplete;
				
				if(pdata.quests[questIndex-1].isFailed) then
					questTag = "Failed";
					level = 0;
				end

				return questLogTitleText, level, questTag, isHeader, isCollapsed, isComplete;
			end
		end
	end
	return PlayerQuest.overrides.GetQuestLogTitle(questIndex);
end

------------------------------------------------------
-- SelectQuestLogEntry(questIndex)					--
--  Returns: None									--
------------------------------------------------------
function SelectQuestLogEntry(questIndex)
	if(PlayerQuest.Ready()) then
		local pdata = PlayerQuest.PlayerData();
		if(pdata.nQuests > 0) then
			if(questIndex > 1 + pdata.nQuests) then
				PlayerQuest.questlog.Selection = 0;
				return PlayerQuest.overrides.SelectQuestLogEntry(questIndex - (1 + pdata.nQuests));
			else
				PlayerQuest.questlog.Selection = questIndex - 1;
				return;
			end
		end
	end
	
	PlayerQuest.questlog.Selection = 0;

	return PlayerQuest.overrides.SelectQuestLogEntry(questIndex);
end

------------------------------------------------------
-- GetNumQuestLeaderBoards(questIndex)				--
--  Returns: number of leaderboards					--
------------------------------------------------------
function GetNumQuestLeaderBoards(test)
	if(PlayerQuest.Ready()) then
		local pdata = PlayerQuest.PlayerData();
		if(pdata.nQuests > 0) then
			if(PlayerQuest.questlog.Selection > 0) then
				local sq = PlayerQuest.questlog.GetSelectedQuest();
				return sq.nLeaderBoards;
			else
				if(test) then
					if(test <= 1+pdata.nQuests) then
						if(test == 0) then
							return 0;
						end
						return pdata.quests[test-1].nLeaderBoards;
					else
						return PlayerQuest.overrides.GetNumQuestLeaderBoards(test-(pdata.nQuests + 1));
					end
				else
					return PlayerQuest.overrides.GetNumQuestLeaderBoards();
				end
			end
		end
	end

	return PlayerQuest.overrides.GetNumQuestLeaderBoards(test);
end

------------------------------------------------------
-- GetQuestLogLeaderBoard(lb, questIndex)			--
--  Returns: leaderboard text						--
------------------------------------------------------
function GetQuestLogLeaderBoard(i, test)
	if(PlayerQuest.Ready()) then
		local pdata = PlayerQuest.PlayerData();
		if(pdata.nQuests > 0) then
			if(PlayerQuest.questlog.Selection > 0) then
				local sq = PlayerQuest.questlog.GetSelectedQuest();
				local a, b, c = PlayerQuest.questlog.PrintLeaderBoard(i), sq.leaderBoards[i].type, sq.leaderBoards[i].isComplete;
				return a, b, c;	
			else
				if(test) then
					if(test <= (pdata.nQuests + 1)) then
						if(test==0) then
							return "", "", nil;
						else
							local sq = pdata.quests[test-1];
							return PlayerQuest.questlog.PrintLeaderBoard(i, test-1), sq.leaderBoards[i].type, sq.leaderBoards[i].isComplete;
						end
					else
						return PlayerQuest.overrides.GetQuestLogLeaderBoard(i, test-(pdata.nQuests + 1));
					end
				else
					return PlayerQuest.overrides.GetQuestLogLeaderBoard(i);
				end
			end
		end
	end

	return PlayerQuest.overrides.GetQuestLogLeaderBoard(i, test);
end

function GetQuestLogTimeLeft()
	if(PlayerQuest.Ready()) then
		local pdata = PlayerQuest.PlayerData();
		if(pdata.nQuests > 0) then
			if(PlayerQuest.questlog.Selection > 0) then
				local sq = PlayerQuest.questlog.GetSelectedQuest();
				return sq.timeLeft;
			end
		end
	end

	return PlayerQuest.overrides.GetQuestLogTimeLeft();
end

----------------------------------------------
-- GetQuestLogRequiredMoney()				--
--  Returns: Required Money					--
----------------------------------------------
function GetQuestLogRequiredMoney()
	if(PlayerQuest.Ready()) then
		local pdata = PlayerQuest.PlayerData();
		if(pdata.nQuests > 0) then
			if(PlayerQuest.questlog.Selection > 0) then
				local sq = PlayerQuest.questlog.GetSelectedQuest();
				return sq.requiredMoney;
			end
		end
	end

	return PlayerQuest.overrides.GetQuestLogRequiredMoney();
end

------------------------------------------------------
-- GetQuestLogItemLink(type, id)					--
--  Returns: A link to the item						--
--  Notes: Not sure how this works, will have to 	--
--	investigate										--
------------------------------------------------------
function GetQuestLogItemLink(type, id)
	if(PlayerQuest.Ready()) then
		local pdata = PlayerQuest.PlayerData();
		if(pdata.nQuests > 0) then
			if(PlayerQuest.questlog.Selection > 0) then
				local sq = PlayerQuest.questlog.GetSelectedQuest();
				return sq.reward[id].link;
			end
		end
	end
	
	return PlayerQuest.overrides.GetQuestLogItemLink(type, id);
end



--------------------------------------------------------
-- GetNumQuestLogRewards;							  --
--  Returns: 										  --
--------------------------------------------------------
function GetNumQuestLogRewards()
	if(PlayerQuest.Ready()) then
		local pdata = PlayerQuest.PlayerData();
		if(pdata.nQuests > 0) then
			if(PlayerQuest.questlog.Selection > 0) then
			local sq = PlayerQuest.questlog.GetSelectedQuest();
				local ind=0;
				for i=1, sq.nRewards, 1 do
					if((sq.rewards[i].type ~= "money") and (sq.rewards[i].type ~= "reputation")) then
						ind = ind + 1;
					end
				end
	
				return ind;
			end
		end
	end
	
	return PlayerQuest.overrides.GetNumQuestLogRewards();
end;

--------------------------------------------------------
-- GetNumQuestLogChoices;
--  Returns: 
-- **UNFINISHED**							--
--------------------------------------------------------
function GetNumQuestLogChoices()
	if(PlayerQuest.Ready()) then
		local pdata = PlayerQuest.PlayerData();
		if(pdata.nQuests > 0) then
			if(PlayerQuest.questlog.Selection > 0) then
				local sq = PlayerQuest.questlog.GetSelectedQuest();
				return sq.nChoices;
			end
		end
	end
	
	return PlayerQuest.overrides.GetNumQuestLogChoices();
end;

--------------------------------------------------------
-- GetQuestLogRewardMoney()							  --
--  Returns: The amount of money rewarded			  --
-- **FINISHED**										  --
--------------------------------------------------------
function GetQuestLogRewardMoney()
	if(PlayerQuest.Ready()) then
		local pdata = PlayerQuest.PlayerData();
		if(pdata.nQuests > 0) then
			if(PlayerQuest.questlog.Selection > 0) then
				local sq = PlayerQuest.questlog.GetSelectedQuest();
				return sq.rewardMoney;
			end
		end
	end
	
	return PlayerQuest.overrides.GetQuestLogRewardMoney();
end;


------------------------------------------------------
-- GetQuestLogChoiceInfo;							--
--  Returns: 
------------------------------------------------------
function GetQuestLogChoiceInfo(id)
	if(PlayerQuest.Ready()) then
		local pdata = PlayerQuest.PlayerData();
		if(pdata.nQuests > 0) then
			if(PlayerQuest.questlog.Selection > 0) then
				local sq = PlayerQuest.questlog.GetSelectedQuest();
				return sq.choices[id].name,
					sq.choices[id].texture,
					sq.choices[id].numItems,
					sq.choices[id].quality,
					sq.choices[id].isUseable;
			end
		end
	end

	return PlayerQuest.overrides.GetQuestLogChoiceInfo(id);
end;

------------------------------------------------------
-- GetQuestLogRewardInfo;
--  Returns: 
------------------------------------------------------
function PlayerQuest.questlog.GetLogRewardID(id)
	if(PlayerQuest.Ready()) then
		local sq = PlayerQuest.questlog.GetSelectedQuest();
		local ind=0;
		for i=1, sq.nRewards, 1 do
			if((sq.rewards[i].type ~= "money") and (sq.rewards[i].type ~= "reputation")) then
				ind = ind + 1;
				if(ind == id) then
					return i;
				end;
			end
		end
	end
end;

function GetQuestLogRewardInfo(id)
	if(PlayerQuest.Ready()) then
		local pdata = PlayerQuest.PlayerData();
		if(pdata.nQuests > 0) then
			if(PlayerQuest.questlog.Selection > 0) then
				local sq = PlayerQuest.questlog.GetSelectedQuest();
				local nid = PlayerQuest.questlog.GetLogRewardID(id);
				return sq.rewards[nid].name,
					sq.rewards[nid].texture,
					sq.rewards[nid].numItems,
					sq.rewards[nid].quality,
					sq.rewards[nid].isUseable;
			end
		end
	end

	return PlayerQuest.overrides.GetQuestLogRewardInfo(id);
end;

--------------------------------------------------------
-- GetQuestLogRewardSpell;
--  Returns: 
--------------------------------------------------------
function GetQuestLogRewardSpell()
	if(PlayerQuest.Ready()) then
		local pdata = PlayerQuest.PlayerData();
		if(pdata.nQuests > 0) then
			if(PlayerQuest.questlog.Selection > 0) then
				return nil;
			end
		end
	end

	return PlayerQuest.overrides.GetQuestLogRewardSpell();
end;

--------------------------------------------------------
-- GetQuestBackgroundMaterial;
--  Returns: 
--------------------------------------------------------
function GetQuestBackgroundMaterial()
	if(PlayerQuest.Ready()) then
		local pdata = PlayerQuest.PlayerData();
		if(PlayerQuest.inter.Intermediate or (PlayerQuest.questlog.Selection > 0)) then
			return nil;
		end
	end
	
	return PlayerQuest.overrides.GetQuestBackgroundMaterial();
end;
