--packages

PlayerQuest.reputation = {};

-- GameTooltip.PQOverride_SetUnit = GameTooltip.SetUnit;

function GameTooltip.SetUnit(gttp, unit)
	gttp:PQOverride_SetUnit(unit);
	if(UnitIsPlayer(unit)) then
		local lastrep, localrep = PlayerQuest.reputation.CheckLastRep(UnitName(unit)), PlayerQuest.reputation.CheckLocalRep(UnitName(unit));
		if(lastrep >= 9.0) then
			lastrep = 8.9;
		elseif(lastrep <= 1.0) then
			lastrep = 1.0;
		end
		if(localrep >= 9.0) then
			localrep = 8.9;
		elseif(localrep <= 1.0) then
			localrep = 1.0;
		end
		
		local lar, lor = floor(lastrep), floor(localrep);
		local lafactionStanding = getglobal("FACTION_STANDING_LABEL"..lar);
		local lofactionStanding = getglobal("FACTION_STANDING_LABEL"..lor);
		
		local guildCheck = GetGuildInfo("player");
		if(guildCheck) then
			if(PlayerQuest.reputation.HasLastRep(UnitName(unit))) then
				gttp:AddLine("Reputation (guild): "..lafactionStanding, FACTION_BAR_COLORS[lar].r,FACTION_BAR_COLORS[lar].g,FACTION_BAR_COLORS[lar].b);
			end
		end
		gttp:AddLine("Reputation (you): "..lofactionStanding, FACTION_BAR_COLORS[lor].r,FACTION_BAR_COLORS[lor].g,FACTION_BAR_COLORS[lor].b);
		gttp:Show()
	end
end


function PlayerQuest.reputation.SetLastRep(player, amount)
	local pdata = PlayerQuest.PlayerData();
	if(pdata.lastRep == nil) then
		pdata.lastRep = {};
	end

	pdata.lastRep[player] = amount;
end

function PlayerQuest.reputation.CheckLastRep(player)
	local pdata = PlayerQuest.PlayerData();
	if(pdata.lastRep == nil) then
		pdata.lastRep = {};
	end
	if(pdata.lastRep[player] == nil) then
		return PlayerQuest.reputation.CheckLocalRep(player); -- Neutral, I believe
	end
	
	return pdata.lastRep[player]
end

function PlayerQuest.reputation.HasLastRep(player)
	local pdata = PlayerQuest.PlayerData();
	if(pdata.lastRep == nil) then
		pdata.lastRep = {};
	end
	if(pdata.lastRep[player] == nil) then
		return nil;
	end
	
	return 1
end

function PlayerQuest.reputation.AddLocalRep(player, amount)
	local pdata = PlayerQuest.PlayerData();
	if(pdata.localRep == nil) then
		pdata.localRep = {};
	end
	if(pdata.localRep[player] == nil) then
		pdata.localRep[player] = 4.0 + amount;
	else
		pdata.localRep[player] = pdata.localRep[player] + amount;
	end
	
	if(pdata.localRep[player] < 1.0) then
		pdata.localRep[player] = 1.0;
	end
	
	local guildName = GetGuildInfo("player");
	
	if(guildName == nil) then
		guildName = PlayerQuest.ShortPlayerName;
	end
	PlayerQuest.comms.Send(player, "MSG_SET_REP_WITH_ME", pdata.localRep[player].."@"..guildName);
end

function PlayerQuest.reputation.CheckLocalRep(player)
	local pdata = PlayerQuest.PlayerData();
	if(pdata.localRep == nil) then
		pdata.localRep = {};
	end
	if(pdata.localRep[player] == nil) then
		return 4.0; -- Neutral, I believe
	end
	
	return pdata.localRep[player]
end

function PlayerQuest.reputation.UpdateRep(player)
	local guildName = GetGuildInfo("player");
	
	if(guildName == nil) then
		guildName = PlayerQuest.ShortPlayerName;
	end

	PlayerQuest.comms.Send(player, "MSG_SET_REP_WITH_ME", PlayerQuest.reputation.CheckLocalRep(player).."@"..guildName);	
end

function PlayerQuest.reputation.UpdateLastRep(player, faction)
	local pdata = PlayerQuest.PlayerData();
	for key, value in pdata.factions do
		if(value.name == faction) then
			local rep = 0.0;
			local nreps = 0.0;
			for key1, value1 in value.players do
				rep = rep + value1;
				nreps = nreps + 1.0;
			end
			rep = rep/nreps;
			
			PlayerQuest.comms.Send(player, "MSG_UPDATE_STORED_REP", rep);
		end
	end	
end

function PlayerQuest.reputation.SetRep(faction, person, amount)
	local pdata = PlayerQuest.PlayerData();
	local found = nil;
	for key, value in pdata.factions do
		--clear all the old factions with that name, just in case...
		if((value.name == person) and (value.name ~= faction)) then
			if(value.players[person] ~= nil) then
				pdata.factions[key] = nil;
			end
		elseif(value.name ~= faction) then
			value.players[person] = nil;
		end
		if(value.name == faction) then
			if(value.players == nil) then
				value.players = {}
			end
			if(value.players[person] == nil) then
				value.players[person] = amount;
			else
				value.players[person] = amount;
			end
			found = 1;
		end
	end
	
	if(found) then
		return;
	end;
	--create a new faction...
	--NOTE: alphabetical sort here?  maybe later...
	pdata.nFactions = pdata.nFactions + 1;
	if(pdata.factions == nil) then
		pdata.factions = {};
	end
	pdata.factions[pdata.nFactions] = {};
	pdata.factions[pdata.nFactions].name = faction;
	pdata.factions[pdata.nFactions].players = {}
	pdata.factions[pdata.nFactions].players[person] = amount;
end


function GetNumFactions()
	if(PlayerQuest.Ready()) then
		local pdata = PlayerQuest.PlayerData();
		if(pdata.nFactions > 0) then
			return PlayerQuest.overrides.GetNumFactions() + 1 + pdata.nFactions;
		end
	end
	
	return PlayerQuest.overrides.GetNumFactions();
end;

--			name, standingID, barValue, atWarWith, canToggleAtWar, isHeader, isCollapsed = GetFactionInfo(factionIndex);
function GetFactionInfo(idx)
	if(PlayerQuest.Ready()) then
		local pdata = PlayerQuest.PlayerData();
		if(idx > PlayerQuest.overrides.GetNumFactions()) then
			local rIdx = idx - 1 - PlayerQuest.overrides.GetNumFactions();
			if(rIdx == 0) then
				return "Guilds", 0, 0, nil, nil, 1, nil;
			end
			--get the info...
			
			local sID, barV;
			local rep = 0.0;
			local nreps = 0.0;
			local name = "None";
			if(pdata.factions[rIdx]) then
				for key, value in pdata.factions[rIdx].players do
					rep = rep + value;
					nreps = nreps + 1.0;
				end
				rep = rep/nreps;
				name = pdata.factions[rIdx].name;
			end
			
			if(rep <= 1.0) then
				sID = 1;
				barV = 0;
			elseif(rep >= 9.0) then
				sID = 8;
				barV = 1;
			else
				sID = math.floor(rep);
				barV = (rep) - sID;
				if(barV>=1.0) then
					barV = 1.0;
				end;
			end		
			
			return name,
				sID,
				barV,
				nil, --pdata.factions[rIdx].atWarWith,
				nil, --pdata.factions[rIdx].canToggleAtWar,
				nil, --pdata.factions[rIdx].isHeader,
				nil; --pdata.factions[rIdx].isCollapsed;
		end
	end
	
	return PlayerQuest.overrides.GetFactionInfo(idx);
end;